package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.LockModeType;
import javax.persistence.Persistence;
import java.time.LocalTime;
import java.util.Arrays;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class PessimisticSharedLocksExample {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) throws Exception {
        ExecutorService es = Executors.newFixedThreadPool(3);
        try {
            persistArticle();

            es.execute(() -> {
                readArticle();
            });

            es.execute(() -> {
                //simulating other user by using different thread
                readArticle();
            });
            es.shutdown();
            es.awaitTermination(1, TimeUnit.MINUTES);
        } finally {
            entityManagerFactory.close();
        }
    }

    private static void readArticle() {
        log("before acquiring read lock");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        Article article = em.find(Article.class, 1L, LockModeType.PESSIMISTIC_READ);
        log("After acquiring read lock", article);
        try {
            TimeUnit.SECONDS.sleep(1);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        em.getTransaction().commit();
        em.close();
        log("Article after read commit", article);
    }

    public static void persistArticle() {
        log("persisting article");
        Article article = new Article("test article");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(article);
        em.getTransaction().commit();
        em.close();
        log("Article persisted", article);
    }

    private static void log(Object... msgs) {
        System.out.println(LocalTime.now() + " - " + Thread.currentThread().getName() +
                " - " + Arrays.toString(msgs));
    }
}